@@ -1,6 +1,6 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989-1994, 2000, 2004, 2006-2014, 2016-2017, 2020-2025
+   Copyright (C) 1989-1994, 2000, 2004, 2006-2014, 2016-2017, 2020-2026
    Free Software Foundation, Inc.
 
    This file is part of GNU M4.
@@ -143,7 +143,7 @@ static builtin const builtin_tab[] = {
   {"undefine", false, false, true, m4_undefine},
   {"undivert", false, false, false, m4_undivert},
 
-  {0, false, false, false, 0},
+  {NULL, false, false, false, NULL},
 
   /* placeholder is intentionally stuck after the table end delimiter,
      so that we can easily find it, while not treating it as a real
