@@ -1,9 +1,9 @@
 /* Test of closein module.
-   Copyright (C) 2007-2011 Free Software Foundation, Inc.
+   Copyright (C) 2007-2026 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
+   the Free Software Foundation, either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program; if not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by Eric Blake.  */
 
@@ -28,8 +27,6 @@
 #include "binary-io.h"
 #include "ignore-value.h"
 
-char *program_name;
-
 /* With no arguments, do nothing.  With arguments, attempt to consume
    first 6 bytes of stdin.  In either case, let exit() take care of
    closing std streams and changing exit status if ferror(stdin).  */
@@ -38,11 +35,10 @@ main (int argc, char **argv)
 {
   char buf[7];
   atexit (close_stdin);
-  program_name = argv[0];
 
   /* close_stdin currently relies on ftell, but mingw ftell is
      unreliable on text mode input.  */
-  SET_BINARY (0);
+  set_binary_mode (0, O_BINARY);
 
   if (argc > 2)
     close (0);
