@@ -1,9 +1,9 @@
 /* Test of xvasprintf() and xasprintf() functions.
-   Copyright (C) 2007-2013 Free Software Foundation, Inc.
+   Copyright (C) 2007-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -12,19 +12,25 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by Bruno Haible <bruno@clisp.org>, 2007.  */
 
 #include <config.h>
 
+/* Tell GCC not to warn about the specific edge cases tested here.  */
+#if _GL_GNUC_PREREQ (4, 3)
+# pragma GCC diagnostic ignored "-Wformat-zero-length"
+# pragma GCC diagnostic ignored "-Wformat-nonliteral"
+# pragma GCC diagnostic ignored "-Wformat-security"
+#endif
+
 #include "xvasprintf.h"
 
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "progname.h"
 #include "macros.h"
 
 static char *
@@ -42,10 +48,9 @@ my_xasprintf (const char *format, ...)
 static void
 test_xvasprintf (void)
 {
-  int repeat;
   char *result;
 
-  for (repeat = 0; repeat <= 8; repeat++)
+  for (int repeat = 0; repeat <= 8; repeat++)
     {
       result = my_xasprintf ("%d", 12345);
       ASSERT (result != NULL);
@@ -81,10 +86,9 @@ test_xvasprintf (void)
 static void
 test_xasprintf (void)
 {
-  int repeat;
   char *result;
 
-  for (repeat = 0; repeat <= 8; repeat++)
+  for (int repeat = 0; repeat <= 8; repeat++)
     {
       result = xasprintf ("%d", 12345);
       ASSERT (result != NULL);
@@ -120,12 +124,10 @@ test_xasprintf (void)
 }
 
 int
-main (int argc _GL_UNUSED, char *argv[])
+main (_GL_UNUSED int argc, char *argv[])
 {
-  set_program_name (argv[0]);
-
   test_xvasprintf ();
   test_xasprintf ();
 
-  return 0;
+  return test_exit_status;
 }
